#!/bin/bash
#
# Author Krigler Pavol
# Version 0.2.0
#
# Bash script Cisco Template Manager (CTM) for checking Cisco configuration
# compared to defined templates.
#
# Copyright 2010 Pavol Krigler <pavol.krigler@gelogic.net>
#
# This file is part of Cisco Template Manager.
#
#    CTM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    CTM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with CTM.  If not, see <http://www.gnu.org/licenses/>.

# CTM configuration file
CONFIGFILE="/etc/ctm/ctm.conf"


#
# Do not modify variables below
#

# Temporary directory where the configuration file is parsed
TMPDIR="$HOME/.ctm-conf-checker-$$/"

GLOBAL_TEMPLATE_FILE="global.template"
GLOBAL_EXCLUDE_TEMPLATE_FILE="global.exclude"

VERSION="0.2.0"

GLOBAL_E=0
GLOBAL_MODE=1
SECTION_MODE=0
SECTION_E=0
IFACE_E=0
SECTION_NUM=0
IFACE_NUM=0

PARSED_GLOBAL="$TMPDIR/globalcfg"

GLOBAL_REPORT="$TMPDIR/report-global"
SECTION_REPORT_PREFIX="report-section"
IFACE_REPORT_PREFIX="report-iface"

# Constants
AVAILABLE_TEMPLATES=1

# 
# Show help
#
function show_help() {
	echo -e "Usage: ctm-conf-checker.sh DEVICE ROLE CONFIG [parseonly]\n"\
"Compare Cisco configuration file with defined template. Version $VERSION\n\n"\
"Mandatory arguments:\n"\
"  DEVICE               select device template to compare\n"\
"  ROLE                 select role of the device\n"\
"  CONFIG               path to Cisco configuration file\n\n"\
"Optional argument:\n"\
"  parseonly            CTM will only parse configuration file to separated files"

if [ $AVAILABLE_TEMPLATES -eq 1 ]; then
	echo
	echo "List of available templates and roles:"
	cd $TEMPLATE_DIR

        for ENTRY in $(ls -1); do
                printf "%-18s" $ENTRY; ls -1 $ENTRY | tr '\n' '/' | sed -e 's/\/$//'
                echo
        done
fi
}

#
# Parse cisco configuration file to separate files (sections)
# global-configuration
# interface(s), Physical-ethernet, Vlan, Loopback
# Return: DEV_HOSTNAME
#
parse_config_file() 
{
	DEV_HOSTNAME=""

	test -n "$DEBUG" && echo "parse_config_file()"

	IFS=$'\n'

	while read LINE
	do
		test -z "$LINE" && continue

		# Detect new Section
		IS_SECTION=$(echo "$LINE" | egrep -c  "^interface |^router |^line |^route-map ")
		if [ $IS_SECTION -eq 1 ]; then
			GLOBAL_MODE=0
			SECTION_MODE=1
			SECTION_NAME=$(echo "$LINE" | tr ' /' '_')
			continue
		fi

#		echo "-->$LINE<--"
		IS_SECTIONEND=$(echo "$LINE" | egrep -c '^!')
#		echo $IS_SECTIONEND

		if [ $IS_SECTIONEND -eq 1 ] && [ $SECTION_MODE -eq 1 ]; then
			test ! -e $TMPDIR/$SECTION_NAME && touch $TMPDIR/$SECTION_NAME && IFACE_NUM=$(($IFACE_NUM+1))
			GLOBAL_MODE=1
			SECTION_MODE=0
		fi

		# GLOBAL configuration
		if [ $GLOBAL_MODE -eq 1 ]; then
			test -z "$DEV_HOSTNAME" && DEV_HOSTNAME=$(echo "$LINE" | grep "hostname " | awk '{print $2}')

			echo "$LINE" | tr -d '\r' >> $PARSED_GLOBAL
			continue
		fi

		# SECTION_MODE configuration
		if [ $SECTION_MODE -eq 1 ]; then
		 	# sed will remove the initial spaces
			echo "$LINE" | sed -e 's/ *\(.*\)/\1/' | tr -d '\r' >> $TMPDIR/$SECTION_NAME
			continue
		fi
	
	done < $TMPDIR/source-config-file

	unset IFS

rm -f $TMPDIR/$1
}

#
# Remove last character '\r' from cisco configuration files
#
remove_trailing_characters()
{
	test -n "$DEBUG" && echo "remove_trailing_characters()"

#	cat $1 | tr -d '\r' > $TMPDIR/source-config-file < $1
	tr -d '\r' > $TMPDIR/source-config-file < $1
}

#
# Check the global configuration settings
#
check_global_settings()
{
	test -n "$DEBUG" && echo "check_global_settings()"

	if [ ! -e "$DEV_TEMPLATE_DIR/$GLOBAL_TEMPLATE_FILE" ]; then
		echo "ERROR: global template configuration file not found in $DEV_TEMPLATE_DIR/$GLOBAL_TEMPLATE_FILE"
		cleanup
		exit
	fi

	while read TEMPLATE_LINE
	do
		# Skip comparation when "!" was found
		if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ] || [ -z "$TEMPLATE_LINE" ]; then 
			continue
		fi
 
		RESULT=$(egrep "$TEMPLATE_LINE" $PARSED_GLOBAL)
		if [ -z "$RESULT" ]; then
			GLOBAL_E=$(($GLOBAL_E+1))
			echo "MISSING: $TEMPLATE_LINE" >> $GLOBAL_REPORT 

			test -n "$DEBUG" && echo "check_global_settings() :: Missing line -> $TEMPLATE_LINE"
		fi
	done <  $DEV_TEMPLATE_DIR/$GLOBAL_TEMPLATE_FILE
}

#
# Check the "exclude" in global configuration settings
#
check_global_exclude_settings()
{
	test -n "$DEBUG" && echo "check_global_exclude_settings()"

	if [ ! -e $DEV_TEMPLATE_DIR/$GLOBAL_EXCLUDE_TEMPLATE_FILE ]; then
		echo "notice: global exclude template configuration file $GLOBAL_EXCLUDE_TEMPLATE_FILE not found"
		return
	fi

	test -n "$DEBUG" && echo "check_global_exclude_settings() :: $DEV_TEMPLATE_DIR/$GLOBAL_EXCLUDE_TEMPLATE_FILE"

	while read TEMPLATE_LINE
	do
		# Skip comparation when "!" or "empty line" was found
		if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ] || [ -z "$TEMPLATE_LINE" ]; then 
			continue
		fi
 
		RESULT=$(egrep "$TEMPLATE_LINE" $PARSED_GLOBAL)
		if [ -n "$RESULT" ]; then
			GLOBAL_E=$(($GLOBAL_E+1))
			echo "$RESULT" | awk '{print "EXCLUDE: " $0}' >> $GLOBAL_REPORT

			test -n "$DEBUG" && echo "check_global_exclude_settings() :: Exclude line -> $RESULT"
		fi
	done <  $DEV_TEMPLATE_DIR/$GLOBAL_EXCLUDE_TEMPLATE_FILE
}

#
# Check sections
#
function check_sections() {
	test -n "$DEBUG" && echo "check_sections()"

	# Check if "sections" directory exists
	if [ ! -e "$DEV_TEMPLATE_DIR/sections" ]; then
		echo "Section subdirectory does not exists, checking skipped"
		return
	fi

	# List all sessions
	SECTION_LIST=$(find "$DEV_TEMPLATE_DIR/sections/" -type f -printf "%f\n" -o -type l -printf "%f\n" | grep -v '.exclude')

#echo "SECTION_LIST=$SECTION_LIST"

	for SEARCH in $SECTION_LIST; do
		test -n "$DEBUG" && echo "check_sections() :: Section file $SEARCH"

		WILDCARD=$(echo $SEARCH | tr '_' '.')

		test -n "$DEBUG" && echo "check_sections() :: find \"$DEV_TEMPLATE_DIR/sections/\" -type f -name \"$WILDCARD\" -printf \"%f\n\""

		RESULT=$(find "$TMPDIR" -type f -regex ".*${WILDCARD}$" -printf "%f\n" -o -type l -regex ".*${WILDCARD}$" -printf "%f\n")

#echo "RESULT=$RESULT"
		for SECTION in $RESULT; do
			SECTION_NUM=$(($SECTION_NUM+1))
	
			if [ ! -e $TMPDIR/$SECTION ]; then
				echo "notice: Section $SECTION not found in configration file, skipping ..."
				continue
			fi
	
			test -n "$DEBUG" && echo "check_sections() :: SECTION=$SECTION"
				
			# Compare template and configuration section 
			while read TEMPLATE_LINE
			do

				# Skip comparation when "!" was found
				if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ] || [ -z "$TEMPLATE_LINE" ]; then 
					continue
				fi

				RESULT=$(egrep "$TEMPLATE_LINE" $TMPDIR/$SECTION)

				if [ -z "$RESULT" ]; then
					SECTION_E=$(($SECTION_E+1))
					echo "MISSING: $TEMPLATE_LINE" >> $TMPDIR/${SECTION_REPORT_PREFIX}-$SECTION
	
					test -n "$DEBUG" && echo "check_sections() :: SECTION=$SECTION :: Missing line -> $TEMPLATE_LINE"
				fi
			done <  $DEV_TEMPLATE_DIR/sections/$SEARCH
		done

	done
}

#
# Check "excluded" commands in sections 
#
function check_sections_exclude() {
	test -n "$DEBUG" && echo "check_sections_exclude()"

	# Check if "sections" directory exists
	if [ ! -e "$DEV_TEMPLATE_DIR/sections" ]; then
#		echo "Section subdirectory does not exists, checking skipped"
		return
	fi

	# List all sessions
	EXC_SECTION_LIST=$(find "$DEV_TEMPLATE_DIR/sections/" -name "*.exclude" -type f -printf "%f\n" | sed -e 's/.exclude$//')

	for SECTION in $EXC_SECTION_LIST; do
		test -n "$DEBUG" && echo "check_sections_exclude() :: Section file $SECTION.exclude"

		if [ ! -e $TMPDIR/$SECTION ]; then
			echo "notice: Section $SECTION not found in configration file, skipping ..."
			continue
		fi
			
		# Compare template and configuration section 
		while read TEMPLATE_LINE
		do
			# Skip comparation when "!" was found
			if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ] || [ -z "$TEMPLATE_LINE" ]; then 
				continue
			fi

			RESULT=$(egrep "$TEMPLATE_LINE" $TMPDIR/$SECTION)

			if [ -n "$RESULT" ]; then
				SECTION_E=$(($SECTION_E+1))
#				echo "EXCLUDE: $RESULT" >> $TMPDIR/${SECTION_REPORT_PREFIX}-$SECTION
				echo "$RESULT" | awk '{print "EXCLUDE: " $0}' >> $TMPDIR/${SECTION_REPORT_PREFIX}-$SECTION

				test -n "$DEBUG" && echo "check_sections_exclude() :: SECTION=$SECTION :: Exclude line -> $RESULT"
			fi
		done <  $DEV_TEMPLATE_DIR/sections/$SECTION.exclude
	done
}

#
# Detect interface description and interface status (up/shutdown)
# Parameter 1: Path to parsed interface
# Return values: IFACE_DESC   -> interface description
#                IFACE_STATUS -> interface status (up/shutdown)
#		 IFACE_TYPE   -> type detected in [] marks
#
function parse_iface() {

	test -n "$DEBUG" && echo "parse_iface()"

	# Check if descriptions exists (take first description)
	IFACE_DESC=$(grep -i "description" $1 | head -1)
	if [ -n "$IFACE_DESC" ]; then

		test -n "$DEBUG" && echo "parse_iface() :: IFACE_DESC=\"$IFACE_DESC\""

		# Detect interface type
		RESULT=$(echo "$IFACE_DESC" | sed -e 's/.*\[\(.*\)\].*/\1/')
		if [ "$RESULT" != "$IFACE_DESC" ]; then
			IFACE_TYPE="$RESULT"
		else
			IFACE_TYPE="N/A"
		fi
	
		test -n "$DEBUG" && echo "parse_iface() :: IFACE_TYPE=$IFACE_TYPE"
	fi

	# Detect if interface is administratively shutdowned
	RESULT=$(egrep -i '^shutdown *$' $1)
	test -z "$RESULT" && IFACE_STATUS="UP" || IFACE_STATUS="DOWN"

	# EXCEPTION for VLAN1
	if [ $(echo $1 | egrep -c "Vlan1$") -eq 1 ]; then 
		IFACE_STATUS="UP"
	fi
	
	test -n "$DEBUG" && echo "parse_iface() :: IFACE_STATUS=$IFACE_STATUS"


	test -n "$DEBUG" && echo "IFACE_STATUS=$IFACE_STATUS, IFACE_DESC=$IFACE_DESC"

	# Check IFACE_TYPE to "N/A" if missing description on interface
	if [ "$IFACE_STATUS" = "UP" ] && [ -z "$IFACE_DESC" ]; then
		IFACE_TYPE="N/A"
	fi
}

#
# Check the service configuration settings on interfaces
# Parameter 1: Absolute path to parsed interface file
# Parameter 2: Template file (absolute path)
#
check_service_settings()
{
	local IFACE_IP_ADDR=""

	test -n "$DEBUG" && echo "check_service_settings()"

	# check if service template and definition file exists
	if [ ! -e "$DEV_TEMPLATE_DIR/services/$2" ]; then
		echo "warning: template file $2 not found, skipping ..."
		return
	fi

	# Check if ".def" file exists (definition)
	if [ ! -e "$DEV_TEMPLATE_DIR/services/$2.def" ]; then
		echo "warning: definition file $2.def not found, skipping ..."
		return
	else
		# Load definitions values (FORMAT and EXAMPLE)
		. $DEV_TEMPLATE_DIR/services/$2.def

		# Chech if values are defined in DEFINITION file
		if [ -z "$FORMAT" ]; then
			echo "ERROR: FORMAT paramater missing in definition file $DEV_TEMPLATE_DIR/services/$2.def, skipping ..."
			return
		fi 
		if [ -z "$EXAMPLE" ]; then
			echo "ERROR: EXAMPLE paramater missing in definition file $DEV_TEMPLATE_DIR/services/$2.def, skipping ..."
			return
		fi 
	fi

	IFACE_NUM=$(($IFACE_NUM+1))


	RESULT=$(echo $IFACE_DESC | egrep "$FORMAT")
	if [ -z "$RESULT" ]; then
		echo "OTHER: Incorrect format of description" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
		echo "OTHER: Your conf: \"$IFACE_DESC\"" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
		echo "OTHER: Should be: \"description $EXAMPLE\"" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
	fi

	while read TEMPLATE_LINE
	do
		test -z "$TEMPLATE_LINE" && continue

		# Skip comparation when "!" was found
		if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ]; then 
			continue
		fi

		# Preparation for reverse DNS 
		if [ -z "$IFACE_IP_ADDR" ]; then
			RESULT=$(egrep "^ip address" $TMPDIR/$1)
			test ! -z "$RESULT" && IFACE_IP_ADDR=$(echo $RESULT | awk '{print $3}')
		#	echo $IFACE_IP_ADDR
		fi

 	
		RESULT=$(egrep "$TEMPLATE_LINE" $TMPDIR/$1)
		if [ -z "$RESULT" ]; then
			IFACE_E=$(($IFACE_E+1))
			echo "MISSING: $TEMPLATE_LINE" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}

			test -n "$DEBUG" && echo "check_service_settings() :: Missing line -> $TEMPLATE_LINE"
		fi
	done <  $DEV_TEMPLATE_DIR/services/$2
}

#
# Check the service configuration settings on interfaces
# Parameter 1: Absolute path to parsed interface file
# Parameter 2: Template file (absolute path)
#
check_service_exclude_settings()
{
	test -n "$DEBUG" && echo "check_service_exclude_settings()"

	# check if service template and definition file exists
	if [ ! -e "$DEV_TEMPLATE_DIR/services/$2.exclude" ]; then
		test -n "$DEBUG" && echo "Exclude file $DEV_TEMPLATE_DIR/services/$2.exclude not found"
		return
	fi

	while read TEMPLATE_LINE
	do
		# Skip comparation when "!" was found
		if [ $(echo "$TEMPLATE_LINE" | grep '!' -c) -eq 1 ]; then 
			continue
		fi

		test -z "$TEMPLATE_LINE" && continue

		RESULT=$(grep "$TEMPLATE_LINE" $TMPDIR/$1)
		if [ -n "$RESULT" ]; then
			IFACE_E=$(($IFACE_E+1))
#			echo "EXCLUDE: $RESULT" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
			echo "$RESULT" | awk '{print "EXCLUDE: " $0}' >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}

			test -n "$DEBUG" && echo "check_service_exclude_settings() :: Exclude line -> $RESULT"
		fi
	done <  $DEV_TEMPLATE_DIR/services/$2.exclude
}

#
# Check all ethernet and serial interfaces/service
#
function check_service_id() {

	test -n "$DEBUG" && echo "check_service_id()"
	
	SEARCH=$(find $TMPDIR -iname "interface*ethernet*" -printf "%f\n" -or -iname "interface*serial*" -printf "%f\n" -or -iname "interface*vlan*" -printf "%f\n")
	for IFACE in $SEARCH; do
		# Detect interface type
		parse_iface $TMPDIR/$IFACE



		# Check if interface has NO description and is NOT shutdowned
		if [ "$IFACE_STATUS" = "UP" ] && [ -z "$IFACE_DESC" ]; then
			IFACE_E=$(($IFACE_E+1))
			echo "MISSING: description and [SERVICE_ID]" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
			continue
		fi

	
		if [ $IFACE_STATUS = "UP" ]; then
			# interface is UP
	
			if [ $IFACE_TYPE != "N/A" ]; then
				# Interface type defined
				check_service_settings $IFACE $IFACE_TYPE 
				check_service_exclude_settings $IFACE $IFACE_TYPE 
			else
				# Interface type not defined
				IFACE_E=$(($IFACE_E+1))

				# Fix interface description from filename "interface_TenGigabitEthernet9_0_0" -> "interface TenGigabitEthernet9/0/0"
				INTERFACE=$(echo $IFACE | sed -e 's/interface_/interface /' | tr '_' '/')

				echo "MISSING: Service ID" >> $TMPDIR/${IFACE_REPORT_PREFIX}-${IFACE}
			fi
		else
			# shutdowned interaface notification
			test -n "$DEBUG" && echo "$IFACE is shutdowned, skipping..."
		fi
		
	done
}


#
# Print report of global configuration
#
function print_global_report() {

	# Print global configuration report
	if [ -e $GLOBAL_REPORT ]; then
		echo -e "\nGLOBAL configuration:\n=====================\n"
	
		MISSING=$(egrep "^MISSING: " $GLOBAL_REPORT)
		if [ -n "$MISSING" ]; then
			echo "Missing command(s):"
			echo "$MISSING" | sed -e 's/MISSING: /     /' | tr -d '^$'
		fi

		EXCLUDE=$(egrep "^EXCLUDE: " $GLOBAL_REPORT)
		if [ -n "$EXCLUDE" ]; then
			echo "Exclude command(s):"
			echo "$EXCLUDE" | sed -e 's/EXCLUDE: /     /' | tr -d '^$'
		fi
	fi
}

#
# Print report of sections
#
function print_section_report() {

	if [ $SECTION_E -gt 0 ]; then
		echo -e "\nSECTION configuration:\n======================\n"
	
		for REPORT in $(find $TMPDIR -name "${SECTION_REPORT_PREFIX}-*" -printf "%f\n"); do
			echo "Section: $(echo $REPORT | sed -e 's/report-section-//' | tr '_' ' ')"
	
			MISSING=$(egrep "^MISSING: " $TMPDIR/$REPORT)
			if [ -n "$MISSING" ]; then
				echo "Missing command(s):"
				echo "$MISSING" | sed -e 's/MISSING: /     /' | tr -d '^$'
			fi
		
			EXCLUDE=$(egrep "^EXCLUDE: " $TMPDIR/$REPORT)
			if [ -n "$EXCLUDE" ]; then
				echo "Exclude command(s):"
				echo "$EXCLUDE" | sed -e 's/EXCLUDE: /     /' | tr -d '^$'
			fi
	
			echo
		done
	fi
}

#
# Print interface report
#
function print_service_id_report() {
	# Print service/interface report(s)
	if [ $IFACE_E -gt 0 ]; then
		echo -e "\nSERVICE configuration:\n======================\n"
	
		for REPORT in $(find $TMPDIR -name "${IFACE_REPORT_PREFIX}-*" -printf "%f\n"); do
			echo "Interface $(echo $REPORT | sed -e 's/report-iface-interface_//' | tr '_' '/')"
	
			MISSING=$(egrep "^MISSING: " $TMPDIR/$REPORT)
			if [ -n "$MISSING" ]; then
				echo "Missing command(s):"
				echo "$MISSING" | sed -e 's/MISSING: /     /' | tr -d '^$'
			fi
		
			EXCLUDE=$(egrep "^EXCLUDE: " $TMPDIR/$REPORT)
			if [ -n "$EXCLUDE" ]; then
				echo "Exclude command(s):"
				echo "$EXCLUDE" | sed -e 's/EXCLUDE: /     /' | tr -d '^$'
			fi
	
			OTHER=$(egrep "^OTHER: " $TMPDIR/$REPORT)
			if [ -n "$OTHER" ]; then
				echo "Other notes:"
				echo "$OTHER" | sed -e 's/OTHER: /     /' | tr -d '^$'
			fi
			echo
		done
	fi
}

#
# Read CTM configuration file
#
function read_config_file() {

	# Read configuration file
	if [ ! -s "$CONFIGFILE" ]; then 
		echo "ERROR: CTM configuration file '$CONFIGFILE' not found or is empty"
		exit 1
	fi
	
	# Read configuration file
	. $CONFIGFILE

	if [ -z "$TEMPLATE_DIR" ] || [ -z "$CHECK_SECTIONS" ]; then
		echo "ERROR: Configuration file '$CONFIGFILE' is incomplete. Missing some arguments."
		exit 1
	fi
}

#
# Check CTM input arguments
#
function check_arguments() {

	# Check Cisco configuration file if exist and has non-zero size
	if [ ! -s "$CTM_CISCO_CONFIG" ]; then 
		echo "ERROR: Cisco configuration file '$CTM_CISCO_CONFIG' not found or is empty"
		exit 1
	fi
	
	# Check if device type directory exists
	if [ ! -e "$TEMPLATE_DIR/$CTM_DEVICE" ]; then 
		echo "ERROR: Device type '$CTM_DEVICE' does not exists. Check template(s) directory $TEMPLATE_DIR/..."
		exit 1
	fi

	# Check if role directory exists
	if [ ! -e "$TEMPLATE_DIR/$CTM_DEVICE/$CTM_ROLE" ]; then 
		echo "ERROR: Device type '$CTM_ROLE' does not exists. Check template(s) directory $TEMPLATE_DIR/$CTM_DEVICE/..."
		exit 1
	fi
}

#
# Cleanup temporary files
#
function cleanup() {
	test -n "$DEBUG" && echo "Cleaning up temporary files"
#	test -e $TMPDIR && rm -f $TMPDIR/*
	test -e $TMPDIR && rm $TMPDIR/*
	rmdir $TMPDIR
}



##############################################################################
#                              MAIN CTM Script                               #
##############################################################################

read_config_file

if [ ! $# -ge 3 ]; then
	show_help
	exit
fi

# Detect mode
if [ "$(echo "$4" | egrep -ic '^parseonly$')" -eq 1 ]; then
	MODE="PARSEONLY"
fi 

CTM_DEVICE="$1"
CTM_ROLE="$2"
CTM_CISCO_CONFIG="$3"

check_arguments

# Create temporary directories before startup
test ! -e $TMPDIR && mkdir $TMPDIR 

remove_trailing_characters $CTM_CISCO_CONFIG

parse_config_file $1

if [ "$MODE" = "PARSEONLY" ]; then
	echo "Configuration file $CTM_CISCO_CONFIG is parsed to $TMPDIR directory" 
	exit
fi

	DEV_TEMPLATE_DIR="$TEMPLATE_DIR/$CTM_DEVICE/$CTM_ROLE"

	if [ ! -e "$DEV_TEMPLATE_DIR" ]; then
		echo "Error: Template directory $DEV_TEMPLATE_DIR does not exist"
		cleanup
		exit 1
	fi

echo "Hostname: $DEV_HOSTNAME, Device type: ${CTM_DEVICE}, Role: ${CTM_ROLE}"

# Check global settings
test $CHECK_GLOBAL -eq 1 && check_global_settings 

# Check for excluded commands
test $CHECK_GLOBAL_EXCLUDE -eq 1 && check_global_exclude_settings 

# Check sections
test $CHECK_SECTIONS -eq 1 && check_sections

# Check "excluded" commands in sections
test $CHECK_SECTIONS_EXCLUDE -eq 1 && check_sections_exclude

# Check interface configuration 
test $CHECK_IFACE_SERVICE_ID -eq 1 && check_service_id


#
# Print reports
#

echo -e "\nGlobal error(s): $GLOBAL_E, Section(s): ${SECTION_NUM} err: ${SECTION_E}, Iface(s): ${IFACE_NUM} err: ${IFACE_E}\n"


print_global_report

print_section_report

print_service_id_report
	

# Remove unused files
cleanup
